"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../utils");
const create_1 = require("./create");
/**
 * Lazy generate Action Creators with Proxy.
 *
 * Fallback to [[createDefaultActionCreators]]
 * if `Proxy` is not supported.
 *
 * @param createActionCreator Retux basic or fsa `createActionCreator` function.
 * @param actionHandlers Retux Action Handlers.
 * @param extraActionCreators Overwrite some of the generated
 * Action Creators or add more.
 */
function proxyDefaultActionCreators(createActionCreator, actionHandlers, extraActionCreators) {
    let cachedOwnKeys;
    const handler = {
        get(memo, type) {
            if (utils_1.hasOwnProperty.call(memo, type)) {
                return memo[type];
            }
            if (extraActionCreators &&
                utils_1.hasOwnProperty.call(extraActionCreators, type)) {
                return (memo[type] = extraActionCreators[type]);
            }
            if (utils_1.hasOwnProperty.call(actionHandlers, type)) {
                return (memo[type] = createActionCreator(type));
            }
        },
        ownKeys() {
            if (!cachedOwnKeys) {
                cachedOwnKeys = new Set([
                    ...Object.keys(actionHandlers),
                    ...Object.keys(extraActionCreators || {})
                ]);
            }
            return [...cachedOwnKeys];
        },
        getOwnPropertyDescriptor(memo, type) {
            if (utils_1.hasOwnProperty.call(memo, type)) {
                return { value: memo[type], configurable: true, enumerable: true };
            }
            if (extraActionCreators &&
                utils_1.hasOwnProperty.call(extraActionCreators, type)) {
                memo[type] = extraActionCreators[type];
                return { value: memo[type], configurable: true, enumerable: true };
            }
            if (utils_1.hasOwnProperty.call(actionHandlers, type)) {
                memo[type] = createActionCreator(type);
                return { value: memo[type], configurable: true, enumerable: true };
            }
        }
    };
    try {
        return new Proxy({}, handler);
    }
    catch (e) {
        return create_1.createDefaultActionCreators(createActionCreator, actionHandlers, extraActionCreators);
    }
}
exports.proxyDefaultActionCreators = proxyDefaultActionCreators;
//# sourceMappingURL=proxy.js.map